#!/usr/bin/perl

use strict;
use warnings;
use File::Basename;
use File::Spec;
use FileHandle;
use POSIX qw{strftime};
use Test;

my $test_num = 1;
my $loaded;
BEGIN { $| = 1; plan (tests => 4);
    print "# Test 1 - Loading the library.\n"}
END {print "not ok 1\n" unless $loaded;}
use Win32API::File::Time qw{GetFileTime SetFileTime utime};
$loaded = 1;
ok ($loaded);

my $me = File::Spec->rel2abs ($0);
my $df = '%d-%b-%Y %H:%M:%S';

print "# This file is $me\n";
my ($patim, $pmtim, $pctim);
(undef, undef, undef, undef, undef, undef, undef, undef,
    $patim, $pmtim, $pctim, undef, undef) = stat ($me);
defined $patim ? pftime ($patim, $pmtim, $pctim) : print <<eod;
# Error - stat $me
#         $!
#         $^E
eod


$test_num++;
my $rslt;
print "# Test $test_num - Get file times.\n";
my ($atime, $mtime, $ctime) = GetFileTime ($me);
$rslt = $mtime == $pmtim && $ctime == $pctim;
ok ($rslt);
$rslt ? pftime ($atime, $mtime, $ctime) : print <<eod;
# GetFileTime failed.
# $^E
eod

my $testfile = File::Spec->catfile ($ENV{TEMP} || $ENV{TMP}, 'Win32API-File-Time.tmp');
my $skip = FileHandle->new (">$testfile") ? '' : "Unable to create $testfile";

$test_num++;
print <<eod;
# Test $test_num - Set the access and modification with SetFileTime.
#     File $testfile
eod
my $now = time () - 5;
($patim, $pmtim, $pctim) = (undef, undef, undef);
$skip or $rslt = SetFileTime ($testfile, $now, $now) and do {
    (undef, undef, undef, undef, undef, undef, undef, undef,
	$patim, $pmtim, $pctim, undef, undef) = stat ($testfile);
    $rslt = $pmtim == $now;	# Don't test atime, because of resolution.
    };
skip ($skip, $rslt);
$skip or $rslt or print <<eod;
# SetFileTime failed.
# $^E
# desired mod time: $now = @{[strftime $df, localtime $now]}.
#  actual mod time: @{[defined $pmtim ? "$pmtim = @{[strftime $df, localtime $pmtim]}" : 'undef']}
eod

$test_num++;
print "# Test $test_num - Set the access and modification with utime.\n";
$now += 5;
($patim, $pmtim, $pctim) = (undef, undef, undef);
$skip or $rslt = utime $now, $now, $testfile and do {
    (undef, undef, undef, undef, undef, undef, undef, undef,
	$patim, $pmtim, $pctim, undef, undef) = stat ($testfile);
    $rslt = $pmtim == $now;	# Don't test atime, because of resolution.
    };
skip ($skip, $rslt);
$skip or $rslt or print <<eod;
# utime failed.
# $^E
eod

$skip or unlink $testfile;

sub pftime {
my ($sat, $smt, $sct) = map {strftime $df, localtime $_} @_;
print <<eod;
# Accessed: $sat
# Modified: $smt
#  Created: $sct
eod
}
sub sftime {
map {strftime $df, localtime $_} @_
}

